/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.sun.comet.example;

import com.sun.grizzly.comet.CometEngine;
import com.sun.grizzly.comet.CometContext;
import com.sun.grizzly.comet.CometHandler;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 *
 */
public class HelloServlet extends HttpServlet {

    private ExecutorService customThreadPool;
    private String contextPath;

    @Override
    public void init(ServletConfig config) throws ServletException {
        customThreadPool = Executors.newCachedThreadPool();

        ServletContext context = config.getServletContext();
        contextPath = context.getContextPath() + "/hello";

        CometEngine engine = CometEngine.getEngine();
        CometContext cometContext = engine.register(contextPath);
        cometContext.setExpirationDelay(30 * 1000);
    }


    @Override
    public void destroy() {
        customThreadPool.shutdown();
        
        super.destroy();
    }


    /**
     * Implements the HTTP GET method.  The GET method is the standard
     * browser method.
     *
     * @param request the request object, containing data from the browser
     * @param repsonse the response object to send data to the browser
     */
    @Override
    public void doGet(HttpServletRequest request,
            HttpServletResponse response)
            throws ServletException, IOException {

        final CometEngine engine = CometEngine.getEngine();
        // Get CometContext associated with the servlet
        final CometContext context = engine.getCometContext(contextPath);

        // Create CometHandler, which will be notified about asynchronous operation progress
        final HelloCometHandler handler = new HelloCometHandler();
        handler.attach(new Context(request, response));

        // Register new CometHandler
        context.addCometHandler(handler);

        // Execute long complex task
        customThreadPool.execute(new LongComplexTask(handler));
    }

    public class LongComplexTask implements Runnable {
        final CometHandler handler;

        public LongComplexTask(CometHandler handler) {
            this.handler = handler;
        }

        public void run() {
            // emulate long task
            try {
                Thread.sleep(5000);
            } catch (Exception e) {}

            CometEngine engine = CometEngine.getEngine();
            CometContext context = engine.getCometContext(contextPath);
            try {
                context.notify("DONE", handler);
            } catch (IOException e) {
                context.removeCometHandler(handler);
            }
        }
        
    }
}
